@@ -25,10 +25,12 @@
 
 #include <math.h>
 
-#include "MathEngine.hh"
+#include "Globals.hh"
+#include "stringAux.hh"
 #include "CharMapper.hh"
 #include "MathMLElement.hh"
 #include "MathMLCharNode.hh"
+#include "MathMLCombinedCharNode.hh"
 #include "RenderingEnvironment.hh"
 
 MathMLCharNode::MathMLCharNode(Char c)
@@ -50,30 +52,29 @@ MathMLCharNode::~MathMLCharNode()
 }
 
 void
-MathMLCharNode::Setup(RenderingEnvironment* env)
+MathMLCharNode::Setup(RenderingEnvironment& env)
 {
-  assert(env != NULL);
-
   delete layout;
   layout = NULL;
 
-  if (env->charMapper.FontifyChar(fChar, env->GetFontAttributes(), ch)) {
+  if (env.charMapper.FontifyChar(fChar, env.GetFontAttributes(), ch)) {
     assert(fChar.font != NULL);
     assert(fChar.charMap != NULL);
 
-#ifdef DEBUG
-    MathEngine::logger(LOG_DEBUG, "successful layout for U+%04x simple index %02x", ch, fChar.nch);
+#if 0
+    env.GetFontAttributes().Dump();
+    Globals::logger(LOG_DEBUG, "successful layout for U+%04x simple index %02x", ch, fChar.nch);
 #endif // DEBUG
   }
 
   FontifiedChar sChar;
 
-  if (env->charMapper.FontifyStretchyChar(sChar, env->GetFontAttributes(), ch)) {
+  if (env.charMapper.FontifyStretchyChar(sChar, env.GetFontAttributes(), ch)) {
     assert(sChar.font != NULL);
     assert(sChar.charMap != NULL);
     
-#ifdef DEBUG
-    MathEngine::logger(LOG_DEBUG, "successful stretchy layout for U+%04x simple index %02x", ch, sChar.nch);
+#if 0
+    Globals::logger(LOG_DEBUG, "successful stretchy layout for U+%04x simple index %02x", ch, sChar.nch);
 #endif // DEBUG
     
     layout = new StretchyCharLayout;
@@ -84,7 +85,7 @@ MathMLCharNode::Setup(RenderingEnvironment* env)
   
   if (fChar.font == NULL && layout == NULL) {
     // no glyph found
-    scaled sppex = env->GetScaledPointsPerEx();
+    scaled sppex = env.GetScaledPointsPerEx();
     box.Set(sppex, (2 * sppex) / 3, sppex / 3);
   }
 }
@@ -96,31 +97,31 @@ MathMLCharNode::SetDefaultLargeGlyph(bool large)
   assert(layout != NULL);
   assert(layout->sChar.font != NULL);
   assert(layout->sChar.charMap != NULL);
-#ifdef DEBUG
-  MathEngine::logger(LOG_DEBUG, "before setting large was %x", layout->sChar.nch);
+#if 0
+  Globals::logger(LOG_DEBUG, "before setting large was %x", layout->sChar.nch);
 #endif // DEBUG
   layout->sChar.nch = layout->sChar.charMap->Map(ch, large);
   fChar = layout->sChar;
-#ifdef DEBUG
-  MathEngine::logger(LOG_DEBUG, "char %x with large %d set to %x", ch, large, layout->sChar.nch);
+#if 0
+  Globals::logger(LOG_DEBUG, "char %x with large %d set to %x", ch, large, layout->sChar.nch);
 #endif // DEBUG
 }
 
 void
-MathMLCharNode::DoLayout()
+MathMLCharNode::DoLayout(const FormattingContext&)
 {
   if (!IsFontified()) return;
 
   fChar.GetBoundingBox(charBox);
   box = charBox;
 
-#ifdef DEBUG
-  MathEngine::logger(LOG_DEBUG, "done char layout for %x resulting in %d height", fChar.nch, sp2ipx(box.GetHeight()));
+#if 1
+  Globals::logger(LOG_DEBUG, "done char layout for %x resulting in %d height", fChar.nch, sp2ipx(box.GetHeight()));
 #endif // DEBUG
 
 #if 1
   if (box.descent > box.ascent && fChar.charMap->GetStretch() != STRETCH_NO) {
-    MathEngine::logger(LOG_DEBUG, "WARNING Texish code here");
+    Globals::logger(LOG_DEBUG, "WARNING Texish code here");
     // BEWARE!
     // vertical stretchy char may have a meaningless bounding box. For example,
     // stretchy chars inside cmex font (for TeX) all have a (quasi) zero ascent.
@@ -140,10 +141,6 @@ MathMLCharNode::DoLayout()
 
     box.ascent = fontAscent - delta;
     box.descent = fontDescent - delta;
-
-    delta = (fontHeight - charBox.GetTotalHeight()) / 2;
-    box.tAscent = fontAscent - delta;
-    box.tDescent = fontDescent - delta;
   }
 #endif
 }
@@ -184,7 +181,7 @@ MathMLCharNode::DoVerticalStretchyLayoutAux(scaled desiredSize, bool)
   for (unsigned i = 0; i < MAX_SIMPLE_CHARS && nch[i] != NULLCHAR; i++) {
     layout->simple = nch[i];
 #if 0
-    MathEngine::logger(LOG_DEBUG, "trying simple char %x for desire %d", layout->simple, sp2ipx(desiredSize));
+    Globals::logger(LOG_DEBUG, "trying simple char %x for desire %d", layout->simple, sp2ipx(desiredSize));
 #endif
     font->CharBox(layout->simple, charBox);
     if (scaledGeq(charBox.GetHeight(), desiredSize)) return;
@@ -345,14 +342,12 @@ MathMLCharNode::DoHorizontalStretchyLayoutAux(scaled desiredSize, bool)
 void
 MathMLCharNode::Render(const DrawingArea& area)
 {
-  if (!HasDirtyChildren()) return;
-
-  assert(GetParent() != NULL);
+  assert(GetParent());
   const GraphicsContext* gc = GetParent()->GetForegroundGC();
 
   if (IsStretchyFontified() && (layout->simple != NULLCHAR || layout->n > 0)) {
-#ifdef DEBUG
-    MathEngine::logger(LOG_DEBUG, "rendering stretchy char U+%04X with simple %02x and n %d", ch, layout->simple, layout->n);
+#if 0
+    Globals::logger(LOG_DEBUG, "rendering stretchy char U+%04X with simple %02x and n %d", ch, layout->simple, layout->n);
 #endif // DEBUG
     if (layout->sChar.charMap->GetStretch() == STRETCH_VERTICAL)
       RenderVerticalStretchyChar(area, gc, GetX(), GetY() + box.descent);
@@ -362,13 +357,11 @@ MathMLCharNode::Render(const DrawingArea& area)
     area.DrawChar(gc, fChar.font, GetX(), GetY() + box.descent - charBox.descent, fChar.nch);
   } else {
     // no glyph available
-    if (MathEngine::DrawMissingCharacter())
+    if (Globals::DrawMissingCharacter())
       RenderMissingCharacter(area, gc);
   }
 
   // area.DrawBoundingBox(gc, GetX(), GetY(), box);
-
-  ResetDirty();
 }
 
 void
@@ -384,8 +377,8 @@ MathMLCharNode::RenderVerticalStretchyChar(const DrawingArea& area,
 
   if (layout->simple != NULLCHAR) {
     y -= charBox.descent;
-#ifdef DEBUG
-    MathEngine::logger(LOG_DEBUG, "rendering vertical stretchy char %x", layout->simple);
+#if 0
+    Globals::logger(LOG_DEBUG, "rendering vertical stretchy char %x", layout->simple);
 #endif // DEBUG
     area.DrawChar(gc, font, x, y, layout->simple);
     return;
@@ -509,12 +502,6 @@ MathMLCharNode::RenderMissingCharacter(const DrawingArea& area, const GraphicsCo
   area.DrawLineToDelta(gc, 0, - box.descent / 2);
 }
 
-bool
-MathMLCharNode::IsChar() const
-{
-  return true;
-}
-
 bool
 MathMLCharNode::IsFontified() const
 {
@@ -555,10 +542,10 @@ MathMLCharNode::GetStretch() const
 }
 
 bool
-MathMLCharNode::CombineWith(const MathMLCharNode* cChar, scaled& shiftX, scaled& shiftY) const
+MathMLCharNode::CombineWith(const Ptr<MathMLCharNode>& cChar, scaled& shiftX, scaled& shiftY) const
 {
-  assert(cChar != NULL);
-  if (!IsFontified() || cChar->IsCombinedChar() || !cChar->IsFontified()) return false;
+  assert(cChar);
+  if (!IsFontified() || is_a<MathMLCombinedCharNode>(cChar) || !cChar->IsFontified()) return false;
   if (!isCombining(cChar->GetChar())) return false;
 
   Char cch = cChar->GetChar();
@@ -569,7 +556,9 @@ MathMLCharNode::CombineWith(const MathMLCharNode* cChar, scaled& shiftX, scaled&
 
   if (isCombiningOverlay(cch)) {
     shiftX = box.lBearing - cBox.lBearing + (box.rBearing - box.lBearing - cBox.rBearing + cBox.lBearing) / 2;
-    shiftY = 0;
+    shiftY = -(cBox.ascent - cBox.descent - box.ascent + box.descent) / 2;
+    //printf("h = %d d = %d h' = %d d' = %d shiftX = %d shiftY = %d\n", sp2ipx(box.ascent), sp2ipx(box.descent),
+    // sp2ipx(cBox.ascent), sp2ipx(cBox.descent), sp2ipx(shiftX), sp2ipx(shiftY));
   } else if (isCombiningBelow(cch)) {
     shiftX = 0;
     shiftY = - box.descent - cBox.ascent;
@@ -602,3 +591,15 @@ MathMLCharNode::CombineWith(const MathMLCharNode* cChar, scaled& shiftX, scaled&
 
   return true;
 }
+
+String*
+MathMLCharNode::GetRawContent() const
+{
+  return allocString(&ch, 1);
+}
+
+unsigned
+MathMLCharNode::GetLogicalContentLength() const
+{
+  return 1;
+}
