@@ -24,15 +24,17 @@
 
 #include "MathMLTextElement.hh"
 
-#if defined(HAVE_MINIDOM)
-MathMLTextElement::MathMLTextElement(mDOMNodeRef node)
-#elif defined(HAVE_GMETADOM)
-MathMLTextElement::MathMLTextElement(const GMetaDOM::Element& node)
-#endif
-  : MathMLTokenElement(node, TAG_MTEXT)
+MathMLTextElement::MathMLTextElement()
 {
 }
 
+#if defined(HAVE_GMETADOM)
+MathMLTextElement::MathMLTextElement(const DOM::Element& node)
+  : MathMLTokenElement(node)
+{
+}
+#endif
+
 MathMLTextElement::~MathMLTextElement()
 {
 }
