@@ -24,66 +24,70 @@
 #include <assert.h>
 #include <stddef.h>
 
-#include "Iterator.hh"
 #include "operatorAux.hh"
 #include "traverseAux.hh"
 #include "MathMLTableElement.hh"
 #include "MathMLOperatorElement.hh"
 #include "MathMLTableCellElement.hh"
+#include "FormattingContext.hh"
 
 void
-MathMLTableElement::DoBoxedLayout(LayoutId id, BreakId bid, scaled availWidth)
+MathMLTableElement::DoLayout(const FormattingContext& ctxt)
 {
-  if (!HasDirtyLayout(id, availWidth)) return;
+  if (DirtyLayout(ctxt))
+    {
+      //cout << "redoing table layout with type " << this << " " << ctxt.GetLayoutType() << endl;
+      scaled aAvailWidth = PrepareLabelsLayout(ctxt);
 
-  scaled aAvailWidth = PrepareLabelsLayout(id, availWidth);
+      if (ctxt.GetLayoutType() == LAYOUT_MIN) DoHorizontalMinimumLayout();
+      else DoHorizontalLayout(ctxt);
 
-  if (id == LAYOUT_MIN) DoHorizontalMinimumLayout();
-  else DoHorizontalLayout(id, bid, aAvailWidth);
+      if (HasLabels()) DoLabelsLayout(ctxt);
 
-  if (HasLabels()) DoLabelsLayout(id, availWidth);
-
-  if (id == LAYOUT_AUTO) {
-    StretchyCellsLayout();
-    AdjustTableWidth(availWidth);
-  }
-
-  DoVerticalLayout(id);
-
-  box.Set(GetTableWidth(), 0, 0);
-  AlignTable(GetTableHeight(), box);
-
-  if (HasLabels()) AdjustTableLayoutWithLabels(id, availWidth);
-
-#if 0
-  cout << "`mtable' DoBoxedLayout (" << id << ',' << bid << ',' << sp2ipx(availWidth);
-  cout << ") [" << sp2ipx(box.width) << ',' << sp2ipx(box.GetHeight()) << ']' << endl;
-#endif
-
-  for (unsigned i = 0; i < nRows; i++) {
-    if (row[i].mtr != NULL)
-      row[i].mtr->box.Set(GetColumnWidth(0, nColumns), row[i].ascent, row[i].descent);
-
-    for (unsigned j = 0; j < nColumns; j++) {
-      if (cell[i][j].mtd != NULL && !cell[i][j].spanned) {
-	scaled width = GetColumnWidth(j, cell[i][j].colSpan);
-	scaled height = GetRowHeight(i, cell[i][j].rowSpan);
-
-	cell[i][j].mtd->box.Set(width, row[i].ascent, height - row[i].ascent);
+      if (ctxt.GetLayoutType() == LAYOUT_AUTO) {
+	StretchyCellsLayout();
+	//AdjustTableWidth(ctxt.GetAvailableWidth());
       }
+
+      DoVerticalLayout(ctxt.GetLayoutType());
+
+      box.Set(GetTableWidth(), 0, 0);
+      AlignTable(GetTableHeight(), box);
+
+      if (HasLabels()) AdjustTableLayoutWithLabels(ctxt);
+
+      for (unsigned i = 0; i < nRows; i++) {
+	if (row[i].mtr)
+	  row[i].mtr->box.Set(GetColumnWidth(0, nColumns), row[i].ascent, row[i].descent);
+
+	for (unsigned j = 0; j < nColumns; j++) {
+	  if (cell[i][j].mtd && !cell[i][j].spanned) {
+	    scaled width = GetColumnWidth(j, cell[i][j].colSpan);
+	    scaled height = GetRowHeight(i, cell[i][j].rowSpan);
+
+	    cell[i][j].mtd->box.Set(width, row[i].ascent, height - row[i].ascent);
+	  }
+	}
+      }
+
+      // because the table invokes directly the layout operations on its
+      // cells, we have to clean the dirty-layout flag on the rows here
+      // otherwise this will stop the propagation mechanism of the dirty
+      // layout flag when one of the rows changes
+      for (std::vector< Ptr<MathMLElement> >::iterator p = content.begin();
+	   p != content.end();
+	   p++)
+	(*p)->ResetDirtyLayout(ctxt);
+
+      ResetDirtyLayout(ctxt);
     }
-  }
-
-  ConfirmLayout(id);
-
-  ResetDirtyLayout(id, availWidth);
 }
 
 void
-MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWidth)
+MathMLTableElement::DoHorizontalLayout(const FormattingContext& ctxt)
 {
   // ok, the available width cannot be negative
-  availWidth = scaledMax(0, availWidth);
+  scaled availWidth = scaledMax(0, ctxt.GetAvailableWidth());
 
   unsigned j;
 
@@ -101,14 +105,14 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
     scaled avail = scaledMax(0, availWidth - GetSpacingWidth());
     scaled availPerColumn = avail / nColumns;
 
-    if (id == LAYOUT_AUTO) {
+    if (ctxt.GetLayoutType() == LAYOUT_AUTO) {
       // in any case all columns must be rendered to the max of the minimum widths!!!!!!
       for (j = 0; j < nColumns; j++)
 	availPerColumn = scaledMax(availPerColumn, column[j].minimumWidth);
     }
 
     // then we render each column with a portion of the available space
-    for (j = 0; j < nColumns; j++) ColumnLayout(j, id, bid, availPerColumn);
+    for (j = 0; j < nColumns; j++) ColumnLayout(j, FormattingContext(ctxt.GetLayoutType(), availPerColumn));
 
     if (nFit == 0 && widthType == WIDTH_AUTO) {
       // oh, the table has no fit columns, so it could be smaller than the
@@ -129,7 +133,7 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
   } else {
     for (j = 0; j < nColumns; j++)
       if (column[j].widthType == COLUMN_WIDTH_FIXED) {
-	ColumnLayout(j, id, BREAK_GOOD, column[j].fixedWidth);
+	ColumnLayout(j, FormattingContext(ctxt.GetLayoutType(), column[j].fixedWidth));
 	column[j].width = scaledMax(column[j].contentWidth, column[j].fixedWidth);
       }
 
@@ -140,7 +144,7 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
       maxTableWidth = float2sp(fixedWidth / (1 - wScale));
     }
 
-    ScaleColumnsLayout(id, (nAuto + nFit == 0) ? BREAK_GOOD : bid, maxTableWidth);
+    ScaleColumnsLayout(FormattingContext(ctxt.GetLayoutType(), maxTableWidth));
     ConfirmHorizontalScaleSpacing(maxTableWidth);
 
 #if 0
@@ -159,7 +163,7 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
 
     for (j = 0; j < nColumns; j++) {
       if (column[j].widthType == COLUMN_WIDTH_AUTO) {
-	ColumnLayout(j, id, bid, avail / n);
+	ColumnLayout(j, FormattingContext(ctxt.GetLayoutType(), avail / n));
 	avail = scaledMax(0, avail - column[j].contentWidth);
 	n--;
       }
@@ -210,7 +214,7 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
 
       for (j = 0; j < nColumns; j++) {
 	if (column[j].widthType == COLUMN_WIDTH_FIT) {
-	  ColumnLayout(j, id, bid, float2sp(sp2float(avail) / n));
+	  ColumnLayout(j, FormattingContext(ctxt.GetLayoutType(), float2sp(sp2float(avail) / n)));
 	  avail = scaledMax(0, avail - scaledMax(column[j].contentWidth, column[j].width));
 	  n--;
 	}
@@ -218,7 +222,7 @@ MathMLTableElement::DoHorizontalLayout(LayoutId id, BreakId bid, scaled availWid
     }
   }
 
-  SpannedCellsLayout(id);
+  SpannedCellsLayout(ctxt);
 }
 
 void
@@ -234,17 +238,17 @@ MathMLTableElement::DoHorizontalMinimumLayout()
 
   for (j = 0; j < nColumns; j++) {
     column[j].minimumWidth = 0;
-    ColumnLayout(j, LAYOUT_MIN, BREAK_GOOD, 0);
+    ColumnLayout(j, FormattingContext(LAYOUT_MIN, 0));
     column[j].minimumWidth = column[j].contentWidth;
   }
 
   for (i = 0; i < nRows; i++) {
     for (j = 0; j < nColumns; j++) {
       unsigned n = cell[i][j].colSpan;
-      if (cell[i][j].mtd != NULL && !cell[i][j].spanned && n > 1) {
+      if (cell[i][j].mtd && !cell[i][j].spanned && n > 1) {
 	scaled minWidth = GetMinimumWidth(j, cell[i][j].colSpan);
-	cell[i][j].mtd->DoBoxedLayout(LAYOUT_MIN, BREAK_GOOD, 0);
-	scaled cellWidth = cell[i][j].mtd->GetMinBoundingBox().width;
+	cell[i][j].mtd->DoLayout(FormattingContext(LAYOUT_MIN, 0));
+	scaled cellWidth = cell[i][j].mtd->GetBoundingBox().width;
 	if (cellWidth > minWidth) {
 	  for (unsigned k = 0; k < n; k++) 
 	    if (minWidth < SP_EPSILON || equalColumns)
@@ -332,7 +336,10 @@ MathMLTableElement::ConfirmHorizontalFixedSpacing()
 
   for (unsigned j = 0; j + 1 < nColumns; j++) {
     if (column[j].spacingType == SPACING_FIXED)
-      column[j].spacing = column[j].fixedSpacing;
+      {
+	//cout << "confirming horizontal fixed spacing " << sp2ipx(column[j].fixedSpacing) << endl;
+	column[j].spacing = column[j].fixedSpacing;
+      }
   }
 }
 
@@ -446,7 +453,7 @@ MathMLTableElement::GetColumnWidth(unsigned j, unsigned n) const
 
   for (unsigned k = 0; k < n; k++) {
     width += column[j].width;
-    if (k < n - 1) width += column[j].spacing;
+    if (k < n - 1) width += column[j + k].spacing;
   }
 
   return width;
@@ -488,64 +495,67 @@ MathMLTableElement::GetContentWidth(ColumnWidthId id) const
 }
 
 void
-MathMLTableElement::ColumnLayout(unsigned j, LayoutId id, BreakId bid, scaled width)
+MathMLTableElement::ColumnLayout(unsigned j, const FormattingContext& ctxt)
 {
   scaled columnWidth = 0;
 
   for (unsigned i = 0; i < nRows; i++) {
     TableCell& tableCell = cell[i][j];
-    if (tableCell.mtd != NULL &&
+    if (tableCell.mtd &&
 	!tableCell.spanned && tableCell.colSpan == 1) {
-      bool stretchyCell = isStretchyOperator(tableCell.mtd->content.GetFirst());
 
-      if (id != LAYOUT_AUTO || !stretchyCell) {
-	// CONFLICT: well, hard to explain. It is obvious that group
-	// alignment must be taken into account here, because we want to
-	// know the width of the column for subsequent computations, but
-	// as we see stretchy cells are delayed. What if an alignment
-	// group is inside a stretchy cell? This is only possible inside
-	// an mrow element with one or more space-like elements (among
-	// which the alignment group(s)) and the stretchy operator. However,
-	// such an alignment group makes little sense, because if the
-	// operator is horizontal then it should stretch to cover all the
-	// available width, so alignment is useless. If the operator is
-	// vertical, then it is usually inside a vertical spanned cell, and
-	// again it is hard to imagine a situation where alignment is useful
-	// for such a cell. For these reasons, we DO NOT consider group
-	// alignment within stretchy cells.
+      if (ctxt.GetLayoutType() != LAYOUT_AUTO || !tableCell.mtd->IsStretchyOperator())
+	{
+	  // CONFLICT: well, hard to explain. It is obvious that group
+	  // alignment must be taken into account here, because we want to
+	  // know the width of the column for subsequent computations, but
+	  // as we see stretchy cells are delayed. What if an alignment
+	  // group is inside a stretchy cell? This is only possible inside
+	  // an mrow element with one or more space-like elements (among
+	  // which the alignment group(s)) and the stretchy operator. However,
+	  // such an alignment group makes little sense, because if the
+	  // operator is horizontal then it should stretch to cover all the
+	  // available width, so alignment is useless. If the operator is
+	  // vertical, then it is usually inside a vertical spanned cell, and
+	  // again it is hard to imagine a situation where alignment is useful
+	  // for such a cell. For these reasons, we DO NOT consider group
+	  // alignment within stretchy cells.
 
-	// first of all we reset all the alignment group so that they have
-	// 0 width for the subsequent layout
-	for (unsigned k = 0; k < tableCell.nAlignGroup; k++) {
-	  assert(tableCell.aGroup[k].group != NULL);
-	  tableCell.aGroup[k].group->SetWidth(0);
+	  // first of all we reset all the alignment group so that they have
+	  // 0 width for the subsequent layout
+	  for (unsigned k = 0; k < tableCell.nAlignGroup; k++)
+	    {
+	      assert(tableCell.aGroup[k].group);
+	      tableCell.aGroup[k].group->SetWidth(0);
+	    }
+
+	  if (tableCell.nAlignGroup > 0)
+	    {
+	      // since this cell is subject to alignment we cannot break its content
+	      tableCell.mtd->DoLayout(FormattingContext(ctxt.GetLayoutType(), width));
+	      tableCell.mtd->CalcGroupsExtent();
+	    } 
+	  else
+	    tableCell.mtd->DoLayout(FormattingContext(ctxt.GetLayoutType(), width));
 	}
 
-	if (tableCell.nAlignGroup > 0) {
-	  // since this cell is subject to alignment we cannot break its content
-	  tableCell.mtd->DoBoxedLayout(id, BREAK_NO, width);
-	  tableCell.mtd->CalcGroupsExtent();
-	} else
-	  tableCell.mtd->DoBoxedLayout(id, bid, width);
-      }
-
       const BoundingBox& cellBox =
-	stretchyCell ?
-	tableCell.mtd->GetMaxBoundingBox() :
+	tableCell.mtd->IsStretchyOperator() ?
+	tableCell.mtd->GetBoundingBox() :
 	tableCell.mtd->GetBoundingBox();
 
       if (cellBox.width > columnWidth) columnWidth = cellBox.width;
     }
   }
 
-  columnWidth = scaledMax(columnWidth, ColumnGroupsLayout(j, id));
+  columnWidth = scaledMax(columnWidth, ColumnGroupsLayout(j, ctxt));
 
   column[j].contentWidth = scaledMax(columnWidth, column[j].minimumWidth);
   column[j].width = scaledMax(column[j].contentWidth, width);
 }
 
 scaled
-MathMLTableElement::ColumnGroupsLayout(unsigned j, LayoutId id)
+MathMLTableElement::ColumnGroupsLayout(unsigned j, const FormattingContext& ctxt)
 {
   unsigned nAlignGroup = column[j].nAlignGroup;
   if (nAlignGroup == 0) return 0;
@@ -558,43 +568,48 @@ MathMLTableElement::ColumnGroupsLayout(unsigned j, LayoutId id)
   for (k = 0; k < nAlignGroup; k++)
     gExtent[k].left = gExtent[k].right = 0;
 
-  for (i = 0; i < nRows; i++) {
-    TableCell& tableCell = cell[i][j];
-    if (tableCell.mtd != NULL &&
-	!tableCell.spanned && tableCell.colSpan == 1) {
-      bool stretchyCell = isStretchyOperator(tableCell.mtd->content.GetFirst());
-
-      if (!stretchyCell) {
-	for (k = 0; k < tableCell.nAlignGroup; k++) {
-	  gExtent[k].left = scaledMax(gExtent[k].left, tableCell.aGroup[k].extent.left);
-	  gExtent[k].right = scaledMax(gExtent[k].right, tableCell.aGroup[k].extent.right);
+  for (i = 0; i < nRows; i++)
+    {
+      TableCell& tableCell = cell[i][j];
+      if (tableCell.mtd &&
+	  !tableCell.spanned &&
+	  tableCell.colSpan == 1 &&
+	  !tableCell.mtd->IsStretchyOperator()) 
+	{
+	  for (k = 0; k < tableCell.nAlignGroup; k++)
+	    {
+	      gExtent[k].left = scaledMax(gExtent[k].left, tableCell.aGroup[k].extent.left);
+	      gExtent[k].right = scaledMax(gExtent[k].right, tableCell.aGroup[k].extent.right);
+	    }
 	}
-      }
     }
-  }
 
   scaled alignedCellWidth = 0;
   for (k = 0; k < nAlignGroup; k++)
     alignedCellWidth += gExtent[k].left + gExtent[k].right;
 
-  if (id == LAYOUT_AUTO) {
-    for (i = 0; i < nRows; i++) {
-      TableCell& tableCell = cell[i][j];
-      if (tableCell.mtd != NULL && !tableCell.spanned && tableCell.colSpan == 1) {
-	bool stretchyCell = isStretchyOperator(tableCell.mtd->content.GetFirst());
-	
-	if (!stretchyCell) {
-	  for (k = 0; k < tableCell.nAlignGroup; k++) {
-	    assert(tableCell.aGroup[k].group != NULL);
+  if (ctxt.GetLayoutType() == LAYOUT_AUTO)
+    {
+      for (i = 0; i < nRows; i++) {
+	TableCell& tableCell = cell[i][j];
+	if (tableCell.mtd &&
+	    !tableCell.spanned &&
+	    tableCell.colSpan == 1 &&
+	    !tableCell.mtd->IsStretchyOperator())
+	  {
+	    for (k = 0; k < tableCell.nAlignGroup; k++)
+	      {
+		assert(tableCell.aGroup[k].group);
 
-	    scaled rightPrev = 0;
-	    if (k > 0) rightPrev = gExtent[k - 1].right - tableCell.aGroup[k - 1].extent.right;
+		scaled rightPrev = 0;
+		if (k > 0) rightPrev = gExtent[k - 1].right - tableCell.aGroup[k - 1].extent.right;
 
-	    tableCell.aGroup[k].group->SetWidth(rightPrev + gExtent[k].left - tableCell.aGroup[k].extent.left);
-	    tableCell.aGroup[k].group->DoBoxedLayout(LAYOUT_AUTO, BREAK_NO, 0);
+		tableCell.aGroup[k].group->SetWidth(rightPrev + gExtent[k].left - tableCell.aGroup[k].extent.left);
+		tableCell.aGroup[k].group->DoLayout(FormattingContext(LAYOUT_AUTO, 0));
+	      }
 	  }
-	}
 
+#if 0
 	// ok, a little explanation: it is not necessary that every cell has exactly the same
 	// number of alignment groups, furthermore it is almost impossibile that even
 	// two cells with the same number of alignment groups have exaclty the same
@@ -604,9 +619,9 @@ MathMLTableElement::ColumnGroupsLayout(unsigned j, LayoutId id)
 	// aligned within their column, then they must have the same width to preserve
 	// alignment inside groups.
 	tableCell.mtd->RecalcBoundingBox(id, alignedCellWidth);
+#endif
       }
     }
-  }
   
   delete [] gExtent;
 
@@ -614,30 +629,30 @@ MathMLTableElement::ColumnGroupsLayout(unsigned j, LayoutId id)
 }
 
 void
-MathMLTableElement::ScaleColumnsLayout(LayoutId id, BreakId bid, scaled tableWidth)
+MathMLTableElement::ScaleColumnsLayout(const FormattingContext& ctxt)
 {
   for (unsigned j = 0; j < nColumns; j++)
     if (column[j].widthType == COLUMN_WIDTH_PERCENTAGE) {
-      ColumnLayout(j, id, bid, float2sp(tableWidth * column[j].scaleWidth));
+      ColumnLayout(j, FormattingContext(ctxt.GetLayoutType(), float2sp(tableWidth * column[j].scaleWidth)));
     }
 }
 
 void
-MathMLTableElement::SpannedCellsLayout(LayoutId id)
+MathMLTableElement::SpannedCellsLayout(const FormattingContext& ctxt)
 {
   for (unsigned i = 0; i < nRows; i++) {
     for (unsigned j = 0; j < nColumns; j++) {
-      if (cell[i][j].mtd != NULL &&
+      if (cell[i][j].mtd &&
 	  !cell[i][j].spanned && cell[i][j].colSpan > 1) {
-	if (id == LAYOUT_MIN) {
-	  cell[i][j].mtd->DoBoxedLayout(LAYOUT_MIN, BREAK_GOOD, 0);
-	  const BoundingBox& cellBox = cell[i][j].mtd->GetMinBoundingBox();
+	if (ctxt.GetLayoutType() == LAYOUT_MIN) {
+	  cell[i][j].mtd->DoLayout(FormattingContext(LAYOUT_MIN, 0));
+	  const BoundingBox& cellBox = cell[i][j].mtd->GetBoundingBox();
 	  scaled widthPerColumn = scaledMax(0, cellBox.width / cell[i][j].colSpan);
 	  for (unsigned k = 0; k < cell[i][j].colSpan; k++)
 	    column[j].minimumWidth = scaledMax(column[j].minimumWidth, widthPerColumn);
 	} else {
 	  scaled spannedWidth = GetColumnWidth(j, cell[i][j].colSpan);
-	  cell[i][j].mtd->DoBoxedLayout(id, BREAK_GOOD, spannedWidth);
+	  cell[i][j].mtd->DoLayout(FormattingContext(ctxt.GetLayoutType(), spannedWidth));
 	}
       }
     }
@@ -647,25 +662,34 @@ MathMLTableElement::SpannedCellsLayout(LayoutId id)
 void
 MathMLTableElement::StretchyCellsLayout()
 {
-  for (unsigned i = 0; i < nRows; i++) {
-    for (unsigned j = 0; j < nColumns; j++) {
-      if (cell[i][j].mtd != NULL && !cell[i][j].spanned) {
-	MathMLOperatorElement* op = findStretchyOperator(cell[i][j].mtd->content.GetFirst());
-	if (op != NULL) {
-	  scaled width = GetColumnWidth(j, cell[i][j].colSpan);
+  for (unsigned i = 0; i < nRows; i++)
+    {
+      for (unsigned j = 0; j < nColumns; j++)
+	{
+	  if (cell[i][j].mtd && !cell[i][j].spanned)
+	    {
+	      Ptr<MathMLElement> cellElem = cell[i][j].mtd->GetChild();
+	      assert(cellElem);
+	      Ptr<MathMLOperatorElement> op = findStretchyOperator(cellElem);
+	      if (op)
+		{
+		  scaled width = GetColumnWidth(j, cell[i][j].colSpan);
 
-	  if (op->GetStretch() == STRETCH_VERTICAL) {
-	    scaled height = GetRowHeight(i, cell[i][j].rowSpan);
-	    op->VerticalStretchTo(row[i].ascent, height - row[i].ascent);
-	  } else {
-	    op->HorizontalStretchTo(width);
-	  }
+		  if (op->GetStretch() == STRETCH_VERTICAL)
+		    {
+		      scaled height = GetRowHeight(i, cell[i][j].rowSpan);
+		      op->VerticalStretchTo(row[i].ascent, height - row[i].ascent);
+		    } 
+		  else 
+		    {
+		      op->HorizontalStretchTo(width);
+		    }
 
-	  cell[i][j].mtd->DoBoxedLayout(LAYOUT_AUTO, BREAK_NO, width);
+		  cell[i][j].mtd->DoLayout(FormattingContext(LAYOUT_AUTO, width));
+		}
+	    }
 	}
-      }
     }
-  }
 }
 
 void
@@ -683,36 +707,36 @@ MathMLTableElement::DoVerticalLayout(LayoutId id)
     scaled descent = 0;
 
     for (j = 0; j < nColumns; j++) 
-      if (cell[i][j].mtd != NULL &&
+      if (cell[i][j].mtd &&
 	  !cell[i][j].spanned &&
 	  cell[i][j].rowAlign == ROW_ALIGN_BASELINE) {
 	const BoundingBox& box = cell[i][j].mtd->GetBoundingBox();
-	ascent = scaledMax(ascent, box.tAscent);
-	if (cell[i][j].rowSpan == 1) descent = scaledMax(descent, box.tDescent);
+        ascent = scaledMax(ascent, box.ascent);
+        if (cell[i][j].rowSpan == 1) descent = scaledMax(descent, box.descent);
       }
 
     if (HasLabels()) {
       if (rowLabel[i].labelElement != NULL &&
-	  rowLabel[i].rowAlign == ROW_ALIGN_BASELINE) {
-	const BoundingBox& labelBox = rowLabel[i].labelElement->GetBoundingBox();
-	ascent = scaledMax(ascent, labelBox.tAscent);
-	descent = scaledMax(descent, labelBox.tDescent);
+          rowLabel[i].rowAlign == ROW_ALIGN_BASELINE) {
+        const BoundingBox& labelBox = rowLabel[i].labelElement->GetBoundingBox();
+        ascent = scaledMax(ascent, labelBox.ascent);
+        descent = scaledMax(descent, labelBox.descent);
       }
     }
 
     for (j = 0; j < nColumns; j++)
-      if (cell[i][j].mtd != NULL &&
+      if (cell[i][j].mtd &&
 	  !cell[i][j].spanned && cell[i][j].rowSpan == 1 &&
 	  cell[i][j].rowAlign != ROW_ALIGN_BASELINE) {
 	const BoundingBox& box = cell[i][j].mtd->GetBoundingBox();
-	if (box.GetHeight() > ascent + descent) descent = box.GetTotalHeight() - ascent;
+	if (box.GetHeight() > ascent + descent) descent = box.GetHeight() - ascent;
       }
 
     if (HasLabels()) {
-      if (rowLabel[i].labelElement != NULL &&
+      if (rowLabel[i].labelElement &&
 	  rowLabel[i].rowAlign != ROW_ALIGN_BASELINE) {
 	const BoundingBox& labelBox = rowLabel[i].labelElement->GetBoundingBox();
-	if (labelBox.GetHeight() > ascent + descent) descent = labelBox.GetTotalHeight() - ascent;
+	if (labelBox.GetHeight() > ascent + descent) descent = labelBox.GetHeight() - ascent;
       }
     }
 
@@ -815,46 +839,53 @@ MathMLTableElement::ConfirmVerticalScaleSpacing(scaled tableHeight)
 void
 MathMLTableElement::SpanRowHeight(LayoutId id)
 {
-  for (unsigned j = 0; j < nColumns; j++) {
-    unsigned i = 0;
-    while (i < nRows) {
-      if (cell[i][j].mtd != NULL &&
-	  !cell[i][j].spanned && cell[i][j].rowSpan > 1) {
-	unsigned n = cell[i][j].rowSpan;
-	scaled height  = GetRowHeight(i, n);
-	const BoundingBox& cellBox = cell[i][j].mtd->GetBoundingBox();
+  for (unsigned j = 0; j < nColumns; j++)
+    {
+      unsigned i = 0;
+      while (i < nRows)
+	{
+	  if (cell[i][j].mtd != NULL &&
+	      !cell[i][j].spanned && cell[i][j].rowSpan > 1)
+	    {
+	      unsigned n = cell[i][j].rowSpan;
+	      scaled height  = GetRowHeight(i, n);
+	      const BoundingBox& cellBox = cell[i][j].mtd->GetBoundingBox();
 
-	if (height < cellBox.GetTotalHeight()) {
-	  // the total height of spanned rows is still smaller than the
-	  // height of the single spanning cell. We have to distribute
-	  // additional space among the spanned rows
-	  scaled rest = cellBox.GetTotalHeight() - height;
-	  while (i < n) {
-	    if (i == n - 1) {
-	      // it is the last column, we assign all the remaining space
-	      row[i].descent += rest;
-	    } else {
-	      // we assign a space proportional to the column width
-	      scaled thisRowRest;
-	      if (height > SP_EPSILON)
-		// if all the columns have 0 width we assign an equal amount
-		// of space to each spanned column
-		thisRowRest = (rest * (row[i].GetHeight()));
-	      else
-		thisRowRest = rest / n;
-	      row[i].descent += thisRowRest;
-	      rest -= thisRowRest;
+	      if (height < cellBox.GetHeight())
+		{
+		  // the total height of spanned rows is still smaller than the
+		  // height of the single spanning cell. We have to distribute
+		  // additional space among the spanned rows
+		  scaled rest = cellBox.GetHeight() - height;
+		  //printf("column %d rest is %d\n", j, sp2ipx(rest));
+		  for (unsigned k = 0; k < n; k++)
+		    {
+		      if (k == n - 1)
+			// it is the last column, we assign all the remaining space
+			row[i + k].descent += rest;
+		      else
+			{
+			  // we assign a space proportional to the column height
+			  scaled thisRowRest;
+			  if (height > SP_EPSILON)
+			    // if all the columns have 0 width we assign an equal amount
+			    // of space to each spanned column
+			    thisRowRest = scaledProp(rest, row[i + k].GetHeight(), height);
+			  else
+			    thisRowRest = rest / n;
+			  row[i + k].descent += thisRowRest;
+			  rest -= thisRowRest;
+			}
+		    }
+		}
+	      i += n;
 	    }
+	  else
 	    i++;
-	  }
-	} else
-	  i += n;
-      } else
-	i++;
+	}
     }
-  }
 }
-
+  
 scaled
 MathMLTableElement::GetRowHeight(unsigned i, unsigned n) const
 {
@@ -995,9 +1026,6 @@ MathMLTableElement::AlignTable(scaled height, BoundingBox& box)
   }
 
   box.descent = height - box.ascent;
-
-  box.tAscent = box.ascent;
-  box.tDescent = box.descent;
 }
 
 // PrepareLabelsLayout: this method is for deciding whether the labels
@@ -1005,12 +1033,12 @@ MathMLTableElement::AlignTable(scaled height, BoundingBox& box)
 // deciding that, the method will return the effective available width
 // to render the table.
 scaled
-MathMLTableElement::PrepareLabelsLayout(LayoutId id, scaled availWidth)
+MathMLTableElement::PrepareLabelsLayout(const FormattingContext& ctxt)
 {
   // overlappingLabels = false;
-  scaled aAvailWidth = availWidth;
+  scaled aAvailWidth = ctxt.GetAvailableWidth();
 
-  if (id == LAYOUT_AUTO && HasLabels()) {
+  if (ctxt.GetLayoutType() == LAYOUT_AUTO && HasLabels()) {
     // FIXME: what if the minLabelSpacing is a percentage value?
     assert(minLabelSpacingType == SPACING_FIXED);
     minLabelSpacing = scaledMax(0, minLabelFixedSpacing);
@@ -1041,41 +1069,41 @@ MathMLTableElement::PrepareLabelsLayout(LayoutId id, scaled availWidth)
 scaled
 MathMLTableElement::GetMaxLabelWidth() const
 {  
-  assert(rowLabel != NULL);
+  assert(rowLabel);
 
   scaled width = 0;
   for (unsigned i = 0; i < nRows; i++) {
-    if (rowLabel[i].labelElement != NULL)
-      width = scaledMax(width, rowLabel[i].labelElement->GetMaxBoundingBox().width);
+    if (rowLabel[i].labelElement)
+      width = scaledMax(width, rowLabel[i].labelElement->GetBoundingBox().width);
   }
 
   return width;
 }
 
 void
-MathMLTableElement::DoLabelsLayout(LayoutId id, scaled availWidth)
+MathMLTableElement::DoLabelsLayout(const FormattingContext& ctxt)
 {
-  assert(rowLabel != NULL);
+  assert(rowLabel);
 
-  scaled availForLabels = scaledMax(0, availWidth - GetTableWidth() - minLabelFixedSpacing);
+  scaled availForLabels = scaledMax(0, ctxt.GetAvailableWidth() - GetTableWidth() - minLabelFixedSpacing);
 
   for (unsigned i = 0; i < nRows; i++) {
-    if (rowLabel[i].labelElement != NULL)
-      rowLabel[i].labelElement->DoBoxedLayout(id, BREAK_NO, availForLabels);
+    if (rowLabel[i].labelElement)
+      rowLabel[i].labelElement->DoLayout(FormattingContext(ctxt.GetLayoutType(), availForLabels));
   }
 }
 
 void
-MathMLTableElement::AdjustTableLayoutWithLabels(LayoutId id, scaled availWidth)
+MathMLTableElement::AdjustTableLayoutWithLabels(const FormattingContext& ctxt)
 {
-  assert(rowLabel != NULL);
+  assert(rowLabel);
 
   tableWidth = box.width;
   labelsWidth = 0;
   leftPadding = 0;
 
   for (unsigned i = 0; i < nRows; i++) {
-    if (rowLabel[i].labelElement != NULL) {
+    if (rowLabel[i].labelElement) {
       const BoundingBox& labelBox = rowLabel[i].labelElement->GetBoundingBox();
       labelsWidth = scaledMax(labelsWidth, labelBox.width);
     }
@@ -1083,8 +1111,9 @@ MathMLTableElement::AdjustTableLayoutWithLabels(LayoutId id, scaled availWidth)
 
   tableWidth = box.width;
 
-  if (id == LAYOUT_AUTO && availWidth > labelsWidth + minLabelSpacing + tableWidth) {
-    scaled extra = availWidth - tableWidth;
+  if (ctxt.GetLayoutType() == LAYOUT_AUTO &&
+      ctxt.GetAvailableWidth() > labelsWidth + minLabelSpacing + tableWidth) {
+    scaled extra = ctxt.GetAvailableWidth() - tableWidth;
 
     if (extra > (minLabelSpacing + labelsWidth) * 2) {
       leftPadding = extra / 2;
@@ -1095,7 +1124,7 @@ MathMLTableElement::AdjustTableLayoutWithLabels(LayoutId id, scaled availWidth)
 	leftPadding = extra - minLabelSpacing - labelsWidth;
     }
 
-    box.width = scaledMax(availWidth, tableWidth + labelsWidth + minLabelSpacing);
+    box.width = scaledMax(ctxt.GetAvailableWidth(), tableWidth + labelsWidth + minLabelSpacing);
   } else {
     if (side == TABLE_SIDE_LEFT || side == TABLE_SIDE_LEFTOVERLAP)
       leftPadding = labelsWidth + minLabelSpacing;
